#!/usr/bin/python
"""
Generate the build tree and Makefiles for PyQwt3D.
"""
# Copyright (C) 2003-2008 Gerard Vermeulen
#
# This file is part of PyQwt3D.
#
# PyQwt3D is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyQwt3D is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# In addition, as a special exception, Gerard Vermeulen gives permission
# to link PyQwt3D dynamically with non-free versions of Qt and PyQt,
# and to distribute PyQwt3D in this form, provided that equally powerful
# versions of Qt and PyQt have been released under the terms of the GNU
# General Public License.
#
# If PyQwt3D is dynamically linked with non-free versions of Qt and PyQt,
# PyQwt3D becomes a free plug-in for a non-free program.


import compileall
import glob
import optparse
import os
import pprint
import re
import shutil
import sys
#import traceback

pyvers="%d.%d"%(sys.version_info.major,sys.version_info.minor)
import PyQt5.QtCore
qtconf = PyQt5.QtCore.PYQT_CONFIGURATION
pyqt_mod_dir="/usr/lib/python%s/dist-packages/PyQt5/"%pyvers
qt_inc_dir="/usr/include/x86_64-linux-gnu/qt5/"
pyqt_sip_dir = "/usr/share/sip/PyQt5"
#/usr/share/sip/PyQt4

class Die(Exception):
    def __init__(self, info):
        Exception.__init__(self, info)

    # __init__()

# class Die


try:
    required = 'At least SIP-4.5 and its development tools are required.'
    import sipconfig
except ImportError:
    raise Die(required)
if 0x040500 > sipconfig._pkg_config['sip_version']:
    raise Die(required)
del required


def get_pyqt_configuration(options):
    """Return the PyQt configuration for Qt3 or Qt4
    """

    if options.qt == 5:
        required = 'At least PyQt-5 and its development tools are required.'
        options.qwt3d = 'Qwt3D_Qt5'
        options.opengl = 'OpenGL_Qt5'
        options.excluded_features.append("-x HAS_QT3 -x HAS_QT4")

    options.subdirs.extend([options.qwt3d, options.opengl])

    try:
        configuration = sipconfig.Configuration()
    except AttributeError:
        raise Die(
            'Check if SIP and PyQt or PyQt5 have been installed properly.'
            )

    return configuration

# get_pyqt_configuration()


def compile_qt_program(name, configuration,
                       extra_defines=[],
                       extra_include_dirs=[],
                       extra_lib_dirs=[],
                       extra_libs=[],
                       ):
    """Compile a simple Qt application.

    name is the name of the single source file
    configuration is the pyqtconfig.Configuration()
    extra_defines is a list of extra preprocessor definitions
    extra_include_dirs is a list of extra directories to search for headers
    extra_lib_dirs is a list of extra directories to search for libraries
    extra_libs is a list of extra libraries
    """    
    makefile = sipconfig.ProgramMakefile(configuration, console=True, qt=True, warnings=True)
    
    makefile.extra_defines.extend(extra_defines)
    makefile.extra_include_dirs.extend(extra_include_dirs)
    makefile.extra_lib_dirs.extend(extra_lib_dirs)
    makefile.extra_libs.extend(extra_libs)

    exe, build = makefile.build_command(name)

    # zap a spurious executable
    try:
        os.remove(exe)
    except OSError:
        pass

    os.system(build)

    if not os.access(exe, os.X_OK):
        return None

    if sys.platform != 'win32':
        exe = './' + exe

    return exe

# compile_qt_program()

    
def lazy_copy_file(source, target):
    """Lazy copy a file to another file:
    - check for a SIP time stamp to skip,
    - check if source and target do really differ,
    - copy the source file to the target if they do,
    - return True on copy and False on no copy.
    """
    if not os.path.exists(target):
        shutil.copy2(source, target)
        return True

    sourcelines = open(source).readlines()
    targetlines = open(target).readlines()

    # global length check
    if len(sourcelines) != len(targetlines):
        shutil.copy2(source, target)
        return True
    
    # skip a SIP time stamp 
    if (len(sourcelines) > 3
        and sourcelines[3].startswith(' * Generated by SIP')
        ):
        line = 4
    else:
        line = 0
        
    # line by line check
    while line < len(sourcelines):
        if sourcelines[line] != targetlines[line]:
            shutil.copy2(source, target)
            return True
        line = line + 1
        
    return False

# lazy_copy_file()


def copy_files(sources, directory):
    """Copy a list of files to a directory
    """ 
    for source in sources:
        shutil.copy2(source, os.path.join(directory, os.path.basename(source)))

# copy_files()


def check_numpy(configuration, options, package):
    """See if the NumPy extension has been installed.
    """

    if options.disable_numpy:
        options.excluded_features.append('-x HAS_NUMPY')
        return options

    try:
        import numpy

        # Try to find numpy/arrayobject.h.
        from  numpy.distutils.misc_util import get_numpy_include_dirs
        include_dirs = get_numpy_include_dirs()
        for inc_dir in include_dirs:
            header = os.path.join(inc_dir, 'numpy', 'arrayobject.h')
            if os.access(header, os.F_OK):
                break
        else:
            print(('NumPy has been installed, '
                   'but its headers are not in the standard location.\n'
                   '%s will be build without support for NumPy.\n'
                   '(Linux users may have to install a development package)\n'
                   ) % (package,))
            raise ImportError
        print('Found NumPy-%s.\n' % numpy.__version__)
        options.extra_defines.append('HAS_NUMPY')
        options.extra_include_dirs.extend(include_dirs)
    except ImportError:
        options.excluded_features.append('-x HAS_NUMPY')
        print(('Failed to find NumPy: '
               '%s will be build without support for NumPy.\n'
               ) % (package,))

    return options

# check_numpy()


def check_sip(configuration, options, package):
    """Check if PyQwt3D can be built with SIP.
    """
    version = configuration.sip_version
    version_str = configuration.sip_version_str
    required = '%s requires at least SIP-4.5' % (package,)
    
    print("Found SIP-%s.\n" % version_str)

    if 0x040500 > version:
        raise SystemExit(required)

    options.extra_include_dirs.append(configuration.sip_inc_dir)

    return options

# check_sip()


def check_compiler(configuration, options, package):
    """Check compiler specifics
    """
    print('Do not get upset by error messages in the next 3 compiler checks:')

    makefile = sipconfig.Makefile(configuration=configuration)
    generator = makefile.optional_string('MAKEFILE_GENERATOR', 'UNIX')
    # FIXME: 'MSVC' should be worse than 'MSVC.NET'
    if generator in ['MSVC', 'MSVC.NET']:
        options.extra_cxxflags.extend(['-GR', '-GX'])
    
    program = os.linesep.join([
        r'#include <stddef.h>',
        r'class a { public: void f(size_t); };',
        r'void a::f(%s) {};',
        r'int main() { return 0; }',
        r'',
        ])
    name = "size_t_check.cpp"
    new = [
        '// Automagically generated by configure.py',
        '',
        '// Uncomment one of the following three lines',
        ]

    for ctype in ('unsigned int', 'unsigned long', 'unsigned long long'):
        open(name, "w").write(program % ctype)
        print("Check if 'size_t' and '%s' are the same type:" % ctype)
        if compile_qt_program(name, configuration):
            comment = ''
            print("YES")
        else:
            print("NO")
            comment =  '// '
        new.append('%stypedef %s size_t;' % (comment, ctype))

    new.extend(['',
                '// Local Variables:',
                '// mode: C++',
                '// c-file-style: "stroustrup"',
                '// End:',
                '',
                ])

    new = '\n'.join(new)
    types_sip = os.path.join(os.pardir, 'sip', 'types.sip')
    if os.access(types_sip, os.R_OK):
        old = open(types_sip, 'r').read()
    else:
        old = ''
    if old != new:
        open(types_sip, 'w').write(new)    

    return options

# check_compiler()


def check_os(configuration, options, package):
    """Adapt to different operating systems
    """
    print("Found '%s' operating system:" % os.name)
    print(sys.version)
    print()

    if os.name == 'nt':
        options.extra_defines.append('WIN32')

    return options

# check_os()


def fix_build_file(name, extra_sources, extra_headers, extra_moc_headers):
    """Extend the targets of a SIP build file with extra files 
    """
    
    keys = ('target', 'sources', 'headers', 'moc_headers')
    sbf = {}
    for key in keys:
        sbf[key] = []

    # Parse,
    nr = 0
    for line in open(name, 'r'):
        nr += 1
        if line[0] != '#':
            eq = line.find('=')
            if eq == -1:
                raise SystemExit(
                    '"%s\" line %d: Line must be in the form '
                    '"key = value value...."' % (name, nr))
        key = line[:eq].strip()
        value = line[eq+1:].strip()
        if key in keys:
            sbf[key].append(value)

    # extend,
    sbf['sources'].extend(extra_sources)
    sbf['headers'].extend(extra_headers)
    sbf['moc_headers'].extend(extra_moc_headers)

    # and write.
    output = open(name, 'w')
    for key in keys:
        if sbf[key]:
            output.write('%s = %s\n' % (key, ' '.join(sbf[key])))#, file=output)

# fix_build_file()


def setup_opengl_build(configuration, options, package):
    """Setup the OpenGL extension build
    """

    print('Setup the OpenGL package build.')
    
    build_dir = options.opengl
    tmp_dir = 'tmp-' + build_dir
    build_file = os.path.join(tmp_dir, '%s.sbf' % options.opengl)

    # zap the temporary directory
    try:
        shutil.rmtree(tmp_dir)
    except:
        pass
    # make a clean temporary directory
    try:
        os.mkdir(tmp_dir)
    except:
        raise Die('Failed to create the temporary build directory.')

    pyqt_sip_flags=""
    if options.qt == 5:
        pyqt_sip_flags =  qtconf['sip_flags']

        sipfile = os.path.join(
            os.pardir, "sip", "OpenGL_Qt5_Module.sip")

    # invoke SIP
    print(qtconf['sip_flags'])
    cmd = ' '.join(
        [configuration.sip_bin,
        "-I",pyqt_sip_dir,
         pyqt_sip_flags,
         '-b', build_file,
         '-c', tmp_dir,
         options.jobs,
         options.trace,
         ]
        # SIP assumes POSIX style path separators
        + [sipfile.replace('\\', '/')]
        )

    print('sip invokation:')
    pprint.pprint(cmd)
    if os.path.exists(build_file):
        os.remove(build_file)
    os.system(cmd)
    if not os.path.exists(build_file):
        raise Die('SIP failed to generate the C++ code.')

    # copy lazily to the build directory to speed up recompilation
    if not os.path.exists(build_dir):
        try:
            os.mkdir(build_dir)
        except:
            raise Die('Failed to create the build directory.')

    lazy_copies = 0
    for pattern in ('*.c', '*.cpp', '*.h', '*.py', '*.sbf'):
        for source in glob.glob(os.path.join(tmp_dir, pattern)):
            target = os.path.join(build_dir, os.path.basename(source))
            if lazy_copy_file(source, target):
                print('Copy %s -> %s.' % (source, target))
                lazy_copies += 1
    print('%s file(s) lazily copied.' % lazy_copies)

    # module makefile
    if options.qt == 5:
        # FIXME
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'Qt'))
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'QtCore'))
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'QtGui'))
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'QtWidgets'))
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'QtOpenGL'))
        options.extra_include_dirs.append(qt_inc_dir)
        """makefile = sipconfig.ModuleMakefile(
            configuration = configuration,
            build_file = os.path.basename(build_file),
            dir = build_dir,
            install_dir = options.module_install_path,
            #installs = installs,
            qt = ['QtOpenGL'],
            opengl = 1,
            warnings = 1,
            debug = options.debug,
            )"""
        makefile = sipconfig.SIPModuleMakefile(
            configuration = configuration,
            build_file = os.path.basename(build_file),
            dir = build_dir,
            install_dir = options.module_install_path,
            #installs = installs,
            #qt = ['QtOpenGL'],
            opengl = 1,
            warnings = 1,
            debug = options.debug,
            )

    makefile.extra_cflags.extend(options.extra_cflags)
    makefile.extra_cxxflags.extend(options.extra_cxxflags)
    makefile.extra_defines.extend(options.extra_defines)
    makefile.extra_include_dirs.extend(options.extra_include_dirs)
    makefile.extra_lflags.extend(options.extra_lflags)
    makefile.extra_libs.extend(options.extra_libs)
    makefile.extra_lib_dirs.extend(options.extra_lib_dirs)
    makefile.generate()

# setup_opengl_build()


def nsis():
    """Generate the script for the Nullsoft Scriptable Install System.
    """
    try:
        from numpy.version import version as numpy_version
        from PyQt5.Qt import PYQT_VERSION_STR, QT_VERSION_STR
    except:
        return

    open('PyQwt3D.nsi', 'w').write(open('PyQwt3D.nsi.in').read() % {
        'PYQT_VERSION': PYQT_VERSION_STR,
        'PYTHON_VERSION': '%s.%s' % sys.version_info[:2],
        'QT_VERSION': QT_VERSION_STR,
        'NUMPY_VERSION': numpy_version,
        })

# nsis()


def setup_qwt3d_build(configuration, options, package):
    """Setup the Qwt3D extension build
    """

    print('Setup the Qwt3D package build.')

    # initialize
    build_dir = options.qwt3d
    tmp_dir = "tmp-%s" % options.qwt3d
    build_file = os.path.join(tmp_dir, "qwt3d.sbf")
    #sip_dir = os.path.join(configuration.pyqt_sip_dir, 'Qwt3D')
    sip_dir = os.path.join(pyqt_sip_dir, 'Qwt3D')
    extra_sources = []
    extra_headers = []
    extra_moc_headers = []
    extra_py_files = glob.glob( os.path.join(os.pardir, 'qt5lib', 'PyQt5', 'Qwt3D', '*.py'))

    # do we compile and link the sources of QwtPlot3D into PyQwt3D?
    if options.qwtplot3d_sources:
        # yes, zap all 'qwtplot3d'
        while options.extra_libs.count('qwtplot3d'):
            options.extra_libs.remove('qwtplot3d')
    elif 'qwtplot3d' not in options.extra_libs:
        # no, add 'qwtplot3d' if needed
        options.extra_libs.append('qwtplot3d')
    # This line is added by the Debian 
    while options.extra_libs.count('qwtplot3d'):
        options.extra_libs.remove('qwtplot3d')
 
    # do we also compile and link the sources of zlib into PyQwt3D?
    if options.zlib_sources:
        options.extra_defines.append('HAVE_ZLIB')

    print("Extended options:")
    pprint.pprint(options.__dict__)
    print()
    
    # do we compile and link the sources of QwtPlot3D statically into PyQwt3D?
    if options.qwtplot3d_sources:
        extra_sources += glob.glob(os.path.join(
            options.qwtplot3d_sources, 'src', '*.cpp'))
        extra_sources += glob.glob(os.path.join(
            options.qwtplot3d_sources, '3rdparty', 'gl2ps', '*.c'))
        extra_headers += glob.glob(os.path.join(
            options.qwtplot3d_sources, 'include', '*.h'))
        extra_headers += glob.glob(os.path.join(
            options.qwtplot3d_sources, '3rdparty', 'gl2ps', '*.h'))
        extra_moc_headers = []
        for header in extra_headers:
            text = open(header).read()
            if re.compile(r'^\s*Q_OBJECT', re.M).search(text):
                extra_moc_headers.append(header)

    # do we compile and link the sources of zlib statically into PyQwt3D?
    if options.zlib_sources:
        examples = ('example.c', 'minigzip.c')
        for source in glob.glob(os.path.join(options.zlib_sources, '*.c')):
            if os.path.basename(source) not in examples:
                extra_sources.append(source)
        extra_headers += glob.glob(os.path.join(
            options.zlib_sources, '*.h'))

    # add the interface to the numerical Python extensions
    extra_sources += glob.glob(os.path.join(os.pardir, 'numpy', '*.cpp'))
    extra_headers += glob.glob(os.path.join(os.pardir, 'numpy', '*.h'))

    # add the extra headers which make protected data members accessible
    extra_headers += glob.glob(os.path.join(os.pardir, 'include', '*.h'))

    # put all code into a clean temporary directory
    try:
        shutil.rmtree(tmp_dir)
    except:
        pass
    try:
        os.mkdir(tmp_dir)
    except:
        raise SystemExit("Failed to create the temporary build directory")

    # copy the extra files
    copy_files(extra_sources, tmp_dir)
    copy_files(extra_headers, tmp_dir)
    copy_files(extra_moc_headers, tmp_dir)
    copy_files(extra_py_files, tmp_dir)

    # fix '#include "gl2ps".h' because gl2ps.h got relocated 
    if options.qwtplot3d_sources:
        for source in [os.path.join(tmp_dir, 'qwt3d_io_gl2ps.cpp')]:
            text = open(source).read()
            if -1 != text.find('../3rdparty/gl2ps/'): 
                open(source, 'w').write(text.replace('../3rdparty/gl2ps/', ''))

    pyqt_sip_flags=""
    if options.qt == 5:
        pyqt_sip_flags = qtconf['sip_flags']
        sipfile = os.path.join(
            os.pardir, "sip", "Qwt3D_Qt5_Module.sip")
        
    # invoke SIP
    cmd = " ".join(
        [configuration.sip_bin,
         # SIP assumes POSIX style path separators
         #"-I", os.path.join(os.pardir, "sip").replace("\\", "/"),
         "-I", pyqt_sip_dir.replace("\\", "/"),
        pyqt_sip_flags,
          "-b", build_file,
         "-c", tmp_dir,
         options.jobs,
         options.trace,
         ]
        + options.sip_include_dirs
        + options.excluded_features
        + options.timelines
        # SIP assumes POSIX style path separators
        + [sipfile.replace("\\", "/")]
        )

    print("sip invokation:")
    pprint.pprint(cmd)
    print()

    if os.path.exists(build_file):
        os.remove(build_file)
    os.system(cmd)
    if not os.path.exists(build_file):
        raise SystemExit('SIP failed to generate the C++ code.')

    # FIXME: sip-4.7 does not generate those include files anymore
    for name in [os.path.join(tmp_dir, name) for name in [
        'sip_Qwt3DAxisVector.h',
        'sip_Qwt3DColorVector.h',
        'sip_Qwt3DCellField.h',
        'sip_Qwt3DDoubleVector.h',
        'sip_Qwt3DFreeVector.h',
        'sip_Qwt3DTriple.h',
        'sip_Qwt3DTripleField.h',
        ]]:
        if not os.path.exists(name):
            open(name, 'w')

    # fix the SIP build file
    fix_build_file(build_file,
                   [os.path.basename(f) for f in extra_sources],
                   [os.path.basename(f) for f in extra_headers],
                   [os.path.basename(f) for f in extra_moc_headers])
    
    # Windows fix: resolve the scope of POINTS in enumValues[]
    for source in glob.glob(os.path.join(tmp_dir, '*.cpp')):
        text = open(source).read()
        # sipNm__Qwt3D_POINTS changed between: SIP-4.2 and SIP-4.2.1
        if (-1 != text.find('{sipNm__Qwt3D_POINTS, POINTS')):
                text = text.replace('{sipNm__Qwt3D_POINTS, POINTS',
                                    '{sipNm__Qwt3D_POINTS, Qwt3D::POINTS')
                open(source, 'w').write(text)

    # copy lazily to the build directory to speed up recompilation
    if not os.path.exists(build_dir):
        try:
            os.mkdir(build_dir)
        except:
            raise SystemExit("Failed to create the build directory")

    lazy_copies = 0
    for pattern in ('*.c', '*.cpp', '*.h', '*.py', '*.sbf'):
        for source in glob.glob(os.path.join(tmp_dir, pattern)):
            target = os.path.join(build_dir, os.path.basename(source))
            if lazy_copy_file(source, target):
                print("Copy %s -> %s." % (source, target))
                lazy_copies += 1
    print("%s file(s) lazily copied." % lazy_copies)

    # byte-compile the Python files
    compileall.compile_dir(build_dir, 1, options.module_install_path)

    # files to be installed
    installs = []
    installs.append([[os.path.basename(f) for f in glob.glob(
        os.path.join(build_dir, '*.py*'))], options.module_install_path])
    for option in options.sip_include_dirs:
        # split and undo the POSIX style path separator
        directory = option.split()[-1].replace('/', os.sep)
        if directory.startswith(os.pardir):
            installs.append([[os.path.join(os.pardir, f) for f in glob.glob(
                os.path.join(directory, "*.sip"))], sip_dir])

    # module makefile
    if options.qt == 5:
        # FIXME
        options.extra_include_dirs.append(
            os.path.join(qt_inc_dir, 'Qt'))
        makefile = sipconfig.SIPModuleMakefile(
            configuration = configuration,
            build_file = os.path.basename(build_file),
            dir = build_dir,
            install_dir = options.module_install_path,
            installs = installs,
            #qt = ['QtCore', 'QtGui', 'QtOpenGL'],
            opengl = 1,
            warnings = 1,
            debug = options.debug,
            )
        """        makefile = sipconfig.SIPModuleMakefile(
        configuration = configuration,
        build_file = os.path.basename(build_file),
        dir = build_dir,
        install_dir = options.module_install_path,
        #installs = installs,
        #qt = ['QtOpenGL'],
        opengl = 1,
        warnings = 1,
        debug = options.debug,
        )"""

    makefile.extra_cflags.extend(options.extra_cflags)
    makefile.extra_cxxflags.extend(options.extra_cxxflags)
    makefile.extra_defines.extend(options.extra_defines)
    makefile.extra_include_dirs.extend(options.extra_include_dirs)
    makefile.extra_lflags.extend(options.extra_lflags)
    makefile.extra_libs.extend(options.extra_libs)
    makefile.extra_lib_dirs.extend(options.extra_lib_dirs)
    makefile.generate()
    nsis()

# setup_qwt3d_build()

    
def setup_parent_build(configuration, options):
    """Generate the parent Makefile
    """
    print("Setup the PyQwt3D build.")
     
    sipconfig.ParentMakefile(configuration = configuration, subdirs = options.subdirs).generate()

# setup_parent_build()


def parse_args():
    """Return the parsed options and args from the command line
    """

    usage = (
        'python configure.py [options]'
        '\n\nEach option takes at most one argument, but some options'
        '\naccumulate arguments when repeated. For example, invoke:'
        '\n\n\tpython configure.py -I %s -I %s'
        '\n\nto search the current *and* parent directories for headers.'
        ) % (os.curdir, os.pardir)

    parser = optparse.OptionParser(usage=usage)

    common_options = optparse.OptionGroup(parser, 'Common options')
    common_options.add_option(
        '-5', '--qt5', action='store_const', const=5, dest='qt',
        default=5,
        help=('build for Qt5 and PyQt5 [default Qt5]'))
    common_options.add_option(
        '-Q', '--qwtplot3d-sources', default='', action='store',
        type='string', metavar='/sources/of/qwtplot3d',
        help=('compile and link the QwtPlot3D source files in'
              ' /sources/of/qwtplot3d statically into PyQwt3D'
              ' (required on Windows)'))
    common_options.add_option(
        '-Z', '--zlib-sources', default='', action='store',
        type='string', metavar='/sources/of/zlib',
        help=('compile and link the QwtPlot3D source files in'
              ' /sources/of/zlib statically into PyQwt3D'
              ' (the -Z option is ignored without the -Q option)'))
    common_options.add_option(
        '-D', '--extra-defines', default=[], action='append',
        type='string', metavar='HAVE_ZLIB',
        help=('add an extra preprocessor definition (HAVE_ZLIB enables'
              ' compression of EPS/PDF/PS/SVG output and HAVE_LIBPNG enables'
              ' pixmaps in the SVG output, but both defines are ignored'
              ' without the -Q option)'))
    common_options.add_option(
        '-I', '--extra-include-dirs', default=[], action='append',
        type='string', metavar='/usr/include/qwtplot3d',
        help=('add an extra directory to search for headers'
              ' (the compiler must be able to find the QwtPlot3D headers'
              ' without the -Q option)'))
    common_options.add_option(
        '-L', '--extra-lib-dirs', default=[], action='append',
        type='string', metavar='/usr/lib/qt3/lib',
        help=('add an extra directory to search for libraries'
              ' (the linker must be able to find the QwtPlot3D library'
              ' without the -Q option)'))
    common_options.add_option(
        '-j', '--jobs', default=0, action='store',
        type='int', metavar='N',
        help=('concatenate the SIP generated code into N files'
              ' [default 1 per class] (to speed up make by running '
              ' simultaneous jobs on multiprocessor systems)'))
    common_options.add_option(
        '-l', '--extra-libs', default=[], action='append',
        type='string', metavar='z',
        help=('add an extra library (to link the zlib library, you must'
              ' specify "zlib" or "zlib1" on Windows'
              ' and "z" on POSIX and MacOS/X)'))
    parser.add_option_group(common_options)

    make_options = optparse.OptionGroup(parser, 'Make options')
    make_options.add_option(
        '--debug', default=False, action='store_true',
        help='enable debugging symbols [default disabled]')
    make_options.add_option(
        '--extra-cflags', default=[], action='append',
        type='string', metavar='EXTRA_CFLAG',
        help='add an extra C compiler flag')
    make_options.add_option(
        '--extra-cxxflags', default=[], action='append',
        type='string', metavar='EXTRA_CXXFLAG',
        help='add an extra C++ compiler flag')
    make_options.add_option(
        '--extra-lflags', default=[], action='append',
        type='string', metavar='EXTRA_LFLAG',
        help='add an extra linker flag')
    parser.add_option_group(make_options)

    sip_options = optparse.OptionGroup(parser, 'SIP options')
    sip_options.add_option(
        '-x', '--excluded-features', default=[], action='append',
        type='string', metavar='EXTRA_SENSORY_PERCEPTION',
        help=('add a feature for SIP to exclude'
              ' (normally one of the features in sip/features.sip)'))
    sip_options.add_option(
        '-t', '--timelines', default=[], action='append',
        type='string', metavar='ESP_3_2_1',
        help=('add a timeline for SIP to adapt to a library version'
              ' (normally one of the timeline options in sip/timelines.sip)'))
    sip_options.add_option(
        '--sip-include-dirs', default=[os.path.join(os.pardir, 'sip')],
        action='append', type='string', metavar='SIP_INCLUDE_DIR',
        help='add an extra directory for SIP to search')
    sip_options.add_option(
        '--trace', default=False, action='store_true',
        help=('enable trace of the execution of the bindings'
              ' [default disabled]'))
    parser.add_option_group(sip_options)
    
    detection_options = optparse.OptionGroup(parser, 'Detection options')
    detection_options.add_option(
        '--disable-numarray', default=False, action='store_true',
        help='disable detection and use of numarray [default enabled]')
    detection_options.add_option(
        '--disable-numeric', default=False, action='store_true',
        help='disable detection and use of Numeric [default enabled]')
    detection_options.add_option(
        '--disable-numpy',
        default=False,
        action='store_true',
        help='disable detection and use of NumPy [default enabled]'
        )
    parser.add_option_group(detection_options)

    install_options = optparse.OptionGroup(parser, 'Install options')
    install_options.add_option(
        '--module-install-path', default='', action='store',
        help= 'specify the install directory for the Python modules'
        )
    parser.add_option_group(install_options)

    options, args =  parser.parse_args()
    
    # tweak some of the options to facilitate later processing
    if options.jobs < 1:
        options.jobs = ''
    else:
        options.jobs = '-j %s' % options.jobs
        
    options.excluded_features = [
        ('-x %s' % f) for f in options.excluded_features
        ]

    # SIP assumes POSIX style path separators
    options.sip_include_dirs = [
        ('-I %s' % f).replace('\\', '/') for f in options.sip_include_dirs
    ]
    
    options.timelines = [
        ('-t %s' % t) for t in options.timelines
        ]

    if options.trace:
        options.trace = '-r'
    else:
        options.trace = ''
        
    if options.qwtplot3d_sources == '':
        options.zlib_sources = ''

    if options.trace:
        options.trace = '-r'
        options.extra_defines.append('TRACE_PYQWT3D')
    else:
        options.trace = ''

    options.modules = []
    options.subdirs = []

    return options, args

# parse_args()


def main():
    
    # parse the command line
    options, args = parse_args()
    print("Command line options:")
    pprint.pprint(options.__dict__)
    print()

    configuration = get_pyqt_configuration(options)

    # extend the options
    options = check_sip(configuration, options, 'PyQwt3D')
    options = check_os(configuration, options, 'PyQwt3D')
    #options = check_compiler(configuration, options, 'PyQwt3D')
    options = check_numpy(configuration, options, 'PyQwt3D')
    if not options.module_install_path:
        options.module_install_path = os.path.join(
            pyqt_mod_dir, 'Qwt3D')

    setup_opengl_build(configuration, options, 'PyQwt3D')
    setup_qwt3d_build(configuration, options, 'PyQwt3D')

    # main makefile
    sipconfig.ParentMakefile(
        configuration = configuration,
        subdirs = options.subdirs).generate()

# main()


if __name__ == "__main__":
    main()

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import os
import random
import re
import sys

def hide(html):
    """Hide a chunk of HTML text for dumb robots"""

    list = []
    for c in html:
        if random.randrange(2):
            list.append('&#%s;' % ord(c))
        else:
            list.append('&#%s;' % hex(ord(c))[1:])
    return ''.join(list)

# hide()


def filter(html):
    """Filter all dumb robot readable mail anchors from a HTML text"""

    def replace(m):
        if m.group(1):
            return ('<a %shref="%s">%s</a>' %
                    (m.group(1), hide(m.group(2)), hide(m.group(3))))
        else:
            return ('<a href="%s">%s</a>' %
                    (hide(m.group(2)), hide(m.group(3))))

    # compatible with latex2html mail anchors
    mailRe = re.compile(
        r'<a (class="ulink" )?href="(mailto:.+)"(?:\n *)?>(.+)</a>', re.M)

    return re.sub(mailRe, replace, html)

# filter()


if __name__ == '__main__':
    for name in sys.argv[1:]:
        html = open(name, 'r').read()
        text = filter(html)
        if text != html:
            os.remove(name)
            file = open(name, 'w')
            file.write(text)
            file.close()

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import sys
from math import log
from PyQt4.Qt import QApplication, QCoreApplication, QFont, QFontDatabase
from PyQt4.Qwt3D import Function, RGBA, SurfacePlot, BOX, X1, Y1, Z1

class Rosenbrock(Function):

    def __init__(self, *args):
        Function.__init__(self, *args)

    # __init__()

    def __call__(self, x, y):
        return log((1-x)*(1-x) + 100*(y-x*x)*(y-x*x)) / 8

    # __call__()

# class Rosenbrock


class Plot(SurfacePlot):
    
    def __init__(self, *args):
        SurfacePlot.__init__(self, *args)
        # fonts
        family = QCoreApplication.instance().font().family()
        if 'Verdana' in QFontDatabase().families():
            family = 'Verdana'
        family = 'Courier'
            
        self.coordinates().setLabelFont(family, 14)
        self.coordinates().setNumberFont(family, 12)
            
        self.setTitle('A Simple SurfacePlot Demonstration');
        self.setTitleFont(family, 16, QFont.Bold)
        self.setBackgroundColor(RGBA(1.0, 1.0, 0.6))

        rosenbrock = Rosenbrock(self)

        rosenbrock.setMesh(41, 31)
        rosenbrock.setDomain(-1.73, 1.5, -1.5, 1.5)
        rosenbrock.setMinZ(-10)
        
        rosenbrock.create()

        self.setRotation(30, 0, 15)
        self.setScale(1, 1, 1)
        self.setShift(0.15, 0, 0)
        self.setZoom(0.9)

        axes = self.coordinates().axes # alias
        for axis in axes:
            axis.setMajors(7)
            axis.setMinors(4)
            
        axes[X1].setLabelString('x-axis')
        axes[Y1].setLabelString('y-axis')
        axes[Z1].setLabelString('z-axis')

        self.setCoordinateStyle(BOX);

        self.updateData();
        self.updateGL();

    # __init__()

# class Plot


def make():
    demo = Plot()
    demo.show()
    # Matrox cards on Linux work better with a resize() after show()
    demo.resize(600, 400)
    return demo

# make()


def main(args):
    app = QApplication(args)
    demo = make()
    app.exec_()

# main()


# Admire
if __name__ == '__main__':
    main(sys.argv)


# Local Variables: ***
# mode: python ***
# End: ***




#ezplot is some sugar coating for Qwt3D

# Copyright (C) 2003-2007 Gerard Vermeulen
#
# This file is part of PyQwt3D.
#
# PyQwt3D is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyQwt3D is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# In addition, as a special exception, Gerard Vermeulen gives permission
# to link PyQwt3D dynamically with non-free versions of Qt and PyQt,
# and to distribute PyQwt3D in this form, provided that equally powerful
# versions of Qt and PyQt have been released under the terms of the GNU
# General Public License.
#
# If PyQwt3D is dynamically linked with non-free versions of Qt and PyQt,
# PyQwt3D becomes a free plug-in for a non-free program.

__all__ = ('plot',)

import numpy as np
from PyQt4.Qwt3D._Qwt3D import SurfacePlot, RGBA
from PyQt4.Qwt3D._Qwt3D import X1, X2, X3, X4, Y1, Y2, Y3, Y4, Z1, Z2, Z3, Z4


def tocube(x, y, z):
    """
    For arrays x, y, z, return scale factors sx and sy, so that dz=sx*dx and
    dz=sy*dy, where dx=max(x)-min(x), dy=max(y)-min(y), and dz=max(z)-min(z)
    """
    dx = float(x.max()-x.min())
    dy = float(y.max()-y.min())
    dz = float(z.max()-z.min())

    return dz/dx, dz/dy

# tocube()


# numpy.meshgrid() returns arrays with the wrong shape!
def meshgrid(x, y):
    """
    For vectors x, y with lengths Nx=len(x) and Ny=len(y), return X, Y
    where X and Y are (Nx, Ny) shaped arrays with the elements of x
    and y repeated to fill the matrix

    EG,

      X, Y = meshgrid([1,2,3], [4,5,6,7])

      X =
        1   1   1   1
        2   2   2   2
        3   3   3   3


      Y =
        4   5   6   7
        4   5   6   7
        4   5   6   7
    """
    x = np.asarray(x)
    y = np.asarray(y)
    numRows, numCols = len(x), len(y)  # not reversed, contrary to np.meshgrid
    x = x.reshape(numRows, 1)
    X = x.repeat(numCols, axis=1)

    y = y.reshape(1, numCols)
    Y = y.repeat(numRows, axis=0)
    return X, Y

# meshgrid()


def plot(x, y, function, title='', labels=('x', 'y', 'z')):
    """
    For vectors x and y, return a plot of function(x, y)
    """
    x = np.asarray(x)
    y = np.asarray(y)
    # numpy.meshgrid() produces arrays with the wrong shape!
    z = function(*meshgrid(x, y))
    sx, sy = tocube(x, y, z)

    w = SurfacePlot()
    if title:
        w.setTitle(title)
    w.setBackgroundColor(RGBA(1.0, 1.0, 1.0))
    w.setScale(1.0, 1.0, 1.0)
    w.setRotation(30, 0, 15)
    # antialiasing
    w.coordinates().setLineSmooth(True)
    w.setSmoothMesh(True)

    # Load the data with scaled end points for the x- and y-axis,
    # so that all axes have a similar length (within a factor 2).
    # Rescale the axes limits, so that the axes look correct.
    w.loadFromData(z, sx*x.min(), sx*x.max(), sy*y.min(), sy*y.max())
    axes = w.coordinates().axes
    for axis in (axes[X1],
                 axes[X2],
                 axes[X3],
                 axes[X4]):
        start, stop = axis.limits()
        axis.setLimits(start/sx, stop/sx)
        axis.setMajors(5)
        axis.setMinors(0)
        axis.setLabelString(labels[0])
    for axis in (axes[Y1],
                 axes[Y2],
                 axes[Y3],
                 axes[Y4]):
        start, stop = axis.limits()
        axis.setLimits(start/sy, stop/sy)
        axis.setMajors(5)
        axis.setMinors(0)
        axis.setLabelString(labels[1])
    for axis in (axes[Z1],
                 axes[Z2],
                 axes[Z3],
                 axes[Z4]):
        axis.setMajors(5)
        axis.setMinors(0)
        axis.setLabelString(labels[2])

    w.updateData()
    w.updateGL()
    w.show()
    return w

# plot()


def test():
    def xonly(x, _): return x
    def yonly(_, y): return y
    def saddle(x, y): return x*y
    def rosenbrock(x, y): return (1-x)**2+100*(y-x**2)**2
    def fantasy(x, y): return x*y*np.exp(4-x*x-y*y)

    a = plot(np.linspace(-3, 3), np.linspace(-1, 1), xonly,
             title="f(x, y) = x")
    b = plot(np.linspace(-3, 3), np.linspace(-1, 1), yonly,
             title="f(x, y) = y")
    c = plot(np.linspace(-3, 3), np.linspace(-3, 3), saddle,
             title="f(x, y) = x*y")
    d = plot(np.linspace(-3, 3), np.linspace(-3, 3), rosenbrock,
             title="f(x, y) = (1-x)**2 + 100*(y-x**2)**2")
    e = plot(np.linspace(-2, 2), np.linspace(-2, 2), fantasy,
             title="f(x, y) = x*y*exp(4-x**2-y**2))")
    raw_input('Happy? ')

# test()


# Local Variables: ***
# mode: python ***
# End: ***
    



#!/usr/bin/env python

# This is a trimmed down version of Marc-Andre Lemburg's py2html.py.
# The original code can be found at http://starship.python.net/~lemburg/.
#
# Borrow (or steal?) PyFontify.py from reportlab.lib.

import PyFontify
import sys
  
pattern="""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN">
<html>
<head>
<title>%(source)s</title>
</head>
<body bgcolor=white>
<pre>
%(target)s
</pre>
</body>
</html>
"""

formats = { 'comment': "<font color=blue>%s</font>",
            'identifier': "<font color=red>%s</font>",
            'keyword': "<b>%s</b>",
            'string': "<font color=green>%s</font>" }

def escape_html(text):
        t = (('&','&amp;'), ('<','&lt;'), ('>','&gt;'))
        for x,y in t:
            text = y.join(text.split(x))
        return text

def py2html(source):
    f = open(source)
    text = f.read()
    f.close()
    tags = PyFontify.fontify(text)
    done = 0
    chunks = []
    for tag, start, end, sublist in tags:
        chunks.append(escape_html(text[done:start]))
        chunks.append(formats[tag] % escape_html(text[start:end]))
        done = end
    chunks.append(escape_html(text[done:]))
        
    dict = { 'source' : source, 'target' : ''.join(chunks) }
    f = open(source + '.html', 'w')
    f.write(pattern % dict)
    f.close()

if __name__ == '__main__':

    if len(sys.argv) == 1:
        print("Usage: ./py2html.py files")
        print("\tfiles is a list of Python source files.")
        sys.exit(1)

    for file in sys.argv[1:]:
        py2html(file)



"""Qwt3D -- a Python interface for the QwtPlot3D library.
"""
#
# Copyright (C) 2003-2007 Gerard Vermeulen
#
# This file is part of PyQwt3D.
#
# PyQwt3D is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyQwt3D is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# In addition, as a special exception, Gerard Vermeulen gives permission
# to link PyQwt3D dynamically with non-free versions of Qt and PyQt,
# and to distribute PyQwt3D in this form, provided that equally powerful
# versions of Qt and PyQt have been released under the terms of the GNU
# General Public License.
#
# If PyQwt3D is dynamically linked with non-free versions of Qt and PyQt,
# PyQwt3D becomes a free plug-in for a non-free program.


from _Qwt3D import *

try:
    from ezplot import *
except ImportError, message:
    if 'numpy' in message:
        print 'Install numpy to use ezplot'
    else:
        raise ImportError, message

def save(plot3d, name, format,
         landscape=VectorWriter.OFF,
         textmode=VectorWriter.NATIVE,
         sortmode=VectorWriter.BSPSORT):
    """save a snapshot a Plot3D widget to a file. 
    - plot3d    : the Plot3D widget or a widget with Plot3D widget as child
    - name      : the file name
    - format    : a case-insensitive string indicating the file format
    - landscape : VectorWriter.ON, OFF or AUTO
    - textmode  : VectorWriter.PIXEL, NATIVE, or TEX
    - sortmode  : VectorWriter.NOSORT, SIMPLESORT, or BSPSORT

    PyQwt3D uses GL2PS to support vector formats as EPS, EPS_GZ, PDF, PGF, PS,
    PS_GZ, SVG, and SVG_GZ. It uses Qt to support pixmap formats as GIF, JPEG,
    PNG, and others.
    
    Returns True on success and False on failure.
    """
    format = format.upper()
    if format in ['EPS', 'EPS_GZ',
                  'PDF',
                  'PGF',
                  'PS', 'PS_GZ',
                  'SVG', 'SVG_GZ',
                  ]:
        gl2ps = IO.outputHandler(format)
        if gl2ps:
            gl2ps.setLandscape(landscape)
            gl2ps.setTextMode(textmode)
            gl2ps.setSortMode(sortmode)

    return IO.save(plot3d, name, format)

# save()

# Local Variables: ***
# mode: python ***
# End: ***




#!/usr/bin/env python

import sys
from math import cos, pi, sin
from PyQt4.Qt import QApplication, QCoreApplication, QFont, QFontDatabase, Qt
from PyQt4.Qwt3D._Qwt3D import ParametricSurface, RGBA, SurfacePlot, Triple, NOCOORD

class Sphere(ParametricSurface):

    def __init__(self, *args):
        ParametricSurface.__init__(self, *args)
        self.setMesh(41, 31)
        self.setDomain(0, 2*pi, 0, pi)
        self.setPeriodic(False, False)

    # __init__()

    def __call__(self, u, v):
        r = 1.0
        return Triple(r*cos(u)*sin(v), r*sin(u)*sin(v), r*cos(v))

    # __call__()

# class Sphere

        
class Plot(SurfacePlot):

    def __init__(self, *args):
        SurfacePlot.__init__(self, *args)
        # fonts
        family = QCoreApplication.instance().font().family()
        if 'Verdana' in QFontDatabase().families():
            family = 'Verdana'

        family = 'Courier'
        self.setTitleFont(family, 16, QFont.Bold)
##         self.setTitleColor(RGBA(1.0, 0.0, 0.0))
        
        self.setTitle("A Simple Parametric Surface Demonstration")

        self.setBackgroundColor(RGBA(1.0, 1.0, 0.6))

        sphere = Sphere(self)
        sphere.create()

        self.setRotation(45, 15, 0)
        self.setCoordinateStyle(NOCOORD);
        self.updateData()
        self.updateGL()

    # __init__()

# class Plot


def make():
    demo = Plot()
    demo.show()
    # Matrox cards on Linux work better with a resize() after show()
    demo.resize(600, 400)
    return demo

# make()


def main(args):
    app = QApplication(args)
    demo = make()
    app.exec_()

# main()


# Admire
if __name__ == '__main__':
    main(sys.argv)


# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import inspect
# iqt is part of PyQwt
import Qwt5.iqt
from Qwt3D import Plot3D, save


def walk(p):
    if isinstance(p, Plot3D):
        yield p
    else:
        if p.children() is not None:
            for c in p.children():
                for cc in walk(c):
                    yield cc
        
# def walk()


def main():
    print inspect.getsource(save)

    for demo in ['ParametricSurfaceDemo',
                 'SimplePlot',
                 'TestNumPy',
                 ]:
        result = __import__(demo).make()
        raw_input("Is the demo looking HAPPY? ")

        for format in ('png', 'pdf', 'ps', 'eps', 'svg'):
            print 'Saving %s.%s...' % (demo, format),
            if save(result, '%s.%s' % (demo, format), format):
                print 'success'
            else:
                print 'failure'
                
    for demo in ['AutoSwitch',
                 'EnrichmentDemo',
                 ]:
        result = __import__(demo).make()
        raw_input("Is the demo looking HAPPY? ")

        for i, w in enumerate(walk(result)):
            for format in ('png', 'pdf', 'ps', 'eps', 'svg'):
                print 'Saving %s%s.%s...' % (demo, i, format),
                if save(w, '%s%s.%s' % (demo, i, format), format):
                    print 'success'
                else:
                    print 'failure'
            
# main()

if __name__ == '__main__':
    main()

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

# A Python translation of the "enrichments" example of QwtPlot3D

import sys
from PyQt5.QtCore import QCoreApplication, Qt
from PyQt5.QtWidgets import (QWidget, QApplication, QLabel, QVBoxLayout,
                             QHBoxLayout, QSlider)
#(QApplication, QCheckBox, QComboBox, QDateTimeEdit,
#        QDial, QDialog, QGridLayout, QGroupBox, QHBoxLayout, QLabel, QLineEdit,
#        QProgressBar, QPushButton, QRadioButton, QScrollBar, QSizePolicy,
#        QSlider, QSpinBox, QStyleFactory, QTableWidget, QTabWidget, #QTextEdit,
#        QVBoxLayout, QWidget)
from PyQt5.QtGui import QFont, QFontDatabase
from PyQt5.Qwt3D._Qwt3D import *
from PyQt5.Qwt3D.OpenGL import *

# enable all tracing options of the SIP generated bindings (requires -r option)
if False:
    import sip
    sip.settracemask(0x3f)

# translated from enrichments.cpp

class Bar(VertexEnrichment):

    def __init__(self, radius = 0.0, level = 1.0):
        VertexEnrichment.__init__(self)
        self.configure(radius, level)

    # init()
    
    def clone(self):
        return self

    # clone()
     
    def configure(self, radius, level):
        self.radius = radius
        self.level = level

    # configure()
        
    def drawBegin(self):
        self.diag = self.radius*(self.plot.hull().maxVertex
                                 - self.plot.hull().minVertex).length()
        glLineWidth(0)
        glEnable(GL_POLYGON_OFFSET_FILL)
        glPolygonOffset(1, 1)

    # drawBegin()
     
    def drawEnd(self):
        pass

    # drawEnd()

    def draw(self, pos):
        interval = self.plot.hull().maxVertex.z - self.plot.hull().minVertex.z
        numlevel = self.plot.hull().minVertex.z + self.level * interval
        interval /= 100
        if pos.z > numlevel - interval and pos.z < numlevel + interval:
            Label3D().draw(pos, self.diag, 2*self.diag)
         
        minz = self.plot.hull().minVertex.z

        # FIXME: rgbat = self.plot.dataColor()(pos)
        rgbat = self.plot.dataColor()(pos.x, pos.y, pos.z)
        rgbab = self.plot.dataColor()(pos.x, pos.y, minz)
 
        glBegin(GL_QUADS)
        glColor4d(rgbab.r, rgbab.g, rgbab.b, rgbab.a)
        glVertex3d(pos.x - self.diag, pos.y - self.diag, minz)
        glVertex3d(pos.x + self.diag, pos.y - self.diag, minz)
        glVertex3d(pos.x + self.diag, pos.y + self.diag, minz)
        glVertex3d(pos.x - self.diag, pos.y + self.diag, minz)
        
         
        if pos.z > numlevel - interval and pos.z < numlevel + interval:
            glColor3d(0.7, 0.0, 0.0)
        else:
            glColor4d(rgbat.r, rgbat.g, rgbat.b, rgbat.a)
             
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        
        glColor4d(rgbab.r, rgbab.g,rgbat.b, rgbab.a)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, minz)
        glColor4d(rgbat.r,rgbat.g, rgbat.b, rgbat.a)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        
        glColor4d(rgbab.r, rgbab.g, rgbat.b, rgbab.a)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, minz)
        glColor4d(rgbat.r, rgbat.g, rgbat.b, rgbat.a)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        
        glColor4d(rgbab.r, rgbab.g, rgbat.b, rgbab.a)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, minz)
        glColor4d(rgbat.r, rgbat.g, rgbat.b, rgbat.a)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        
        glColor4d(rgbab.r, rgbab.g, rgbat.b, rgbab.a)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, minz)
        glColor4d(rgbat.r, rgbat.g, rgbat.b, rgbat.a)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glEnd()
         
        glColor3d(0, 0, 0)
        glBegin(GL_LINES)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, minz)
 
        glVertex3d(pos.x-self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        
        glVertex3d(pos.x-self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x-self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y-self.diag, pos.z)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x+self.diag, pos.y+self.diag, pos.z)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, minz)
        glVertex3d(pos.x-self.diag, pos.y+self.diag, pos.z)
        glEnd()

    # draw()

# class Bar


class Label3D:
 
    def __init__(self):
        pass

    # __init__()
     
    def draw(self, pos, w, h):
        gap = 0.3
        glColor3d(1,1,1)
        glBegin(GL_QUADS)
        glVertex3d(pos.x - w, pos.y, pos.z + gap);
        glVertex3d(pos.x + w, pos.y, pos.z + gap);
        glVertex3d(pos.x + w, pos.y, pos.z + gap + h)
        glVertex3d(pos.x - w, pos.y, pos.z + gap + h)
        glEnd()
        glColor3d(0.4,0,0)
        glBegin(GL_LINE_LOOP)
        glVertex3d(pos.x - w, pos.y,pos.z + gap)
        glVertex3d(pos.x + w, pos.y,pos.z + gap)
        glVertex3d(pos.x + w, pos.y,pos.z + gap + h)
        glVertex3d(pos.x - w, pos.y, pos.z + gap + h)
        glEnd()
        glBegin(GL_LINES)
        glVertex3d(pos.x, pos.y, pos.z)
        glVertex3d(pos.x, pos.y, pos.z + gap)
        glEnd()

    # draw()

# class Label3D


class Hat(Function):
 
    def __init__(self, *args):
        Function.__init__(self, *args)

    # __init__()
    
    def __call__(self, x, y):
        return 1.0/(x*x + y*y + 0.5)

    # __call__()

    def name(self):
        return QString('$\\frac{1}{x^2+y^2+\\frac{1}{2}}$')

    # name()

# class Hat


# translated from enrichmentmainwindow.cpp
 
class EnrichmentDemo(QWidget):
     
    def __init__(self, *args):
        QWidget.__init__(self, *args)
        family = QCoreApplication.instance().font().family()

        plot = self.plot = SurfacePlot(self)
        plot.setTitle('Bar Style (Vertex Enrichment)')
        if 'Verdana' in QFontDatabase().families():
            family = 'Verdana'
        family = 'Courier'
            
        plot.setTitleFont(family, 16, QFont.Bold)
        plot.coordinates().setLabelFont(family, 14)
        plot.coordinates().setNumberFont(family, 12)

        plot.setZoom(0.8);
        plot.setRotation(30.0, 0.0, 15.0)

        plot.setCoordinateStyle(BOX)
        self.width = 0.007
        self.level = .5
        self.bar = plot.setPlotStyle(Bar(self.width, self.level))
   
        hat = Hat(plot)
        hat.setMesh(23, 21)
        hat.setDomain(-1.8, 1.7, -1.6, 1.7)
        hat.create()
        
        plot.setFloorStyle(FLOORDATA)
        
        axes = plot.coordinates().axes # alias
        for axis in axes:
            axis.setMajors(5)
            axis.setMinors(4)
             
        plot.coordinates().setGridLinesColor(RGBA(0.0, 0.0, 0.5))
        plot.coordinates().setLineWidth(1)
        plot.coordinates().setNumberFont(family, 8)
        plot.coordinates().adjustNumbers(5)

        self.setColor()
        
        plot.updateData();
        plot.updateGL();

        # Level
        levelLabel = QLabel('Level')
        levelSlider = QSlider(self)
        levelSlider.setValue(50);
        levelSlider.setTickPosition(QSlider.TicksRight)
        levelLayout = QVBoxLayout()
        levelLayout.addWidget(levelLabel)
        levelLayout.addWidget(levelSlider)

        levelSlider.valueChanged.connect(self.setLevel)

        # Layout
        mainLayout = QHBoxLayout()
        self.setLayout(mainLayout)
        mainLayout.addWidget(plot)
        mainLayout.addLayout(levelLayout)
       
    # __init__()

    def setColor(self):
        i, step = 252, 4
        colorVector = ColorVector()
        while (i>=0):
            colorVector.push_back(RGBA(i/255.0, max((i-60)/255.0, 0.0), 0.0))
            step -= 1
            if step == 0:
                i -= 4
                step = 4
        color = StandardColor(self.plot)
        color.setColorVector(colorVector)
        self.plot.setDataColor(color)

    # setColor()

    def setLevel(self, level):
        self.level = 1.0 - 0.01*level;
        self.bar.configure(self.width, self.level)
        self.plot.updateData()
        self.plot.updateGL()

    # setLevel()
 
# class EnrichmentDemo


def make(): 
    demo = EnrichmentDemo()
    demo.show()
    # Matrox cards on Linux work better with a resize() after show()
    demo.resize(600, 400)
    return demo
 
# make()
 
 
def main(args):
    app = QApplication(args)
    demo = make()
    app.exec_()
 
# main()
 
 
# Admire
if __name__ == '__main__':
    main(sys.argv)
    
# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import optparse
import os
import sys

def parse_invokation():
    """Return the parsed options and arguments from the command line
    """

    usage = (
        '%prog [-h] [-t N] directory (one or more file name extensions)'
        )

    parser = optparse.OptionParser(usage=usage)
    parser.add_option(
        '-t', '--tabs', default=2, action='store',
        type='int', metavar='N',
        help='Have tabs N characters apart [default 8]'
        )

    options, arguments = parser.parse_args()

    if len(arguments) < 2:
        parser.print_help()
        sys.exit(1)

    return options, arguments

# parse_invokation()


def process(filename, tabsize):
    """Stolen from Python-X.Y.Z/Tools/scripts/untabify
    """
    try:
        f = open(filename)
        text = f.read()
        f.close()
    except IOError, msg:
        print "%r: I/O error: %s" % (filename, msg)
        return

    newtext = text.replace('\r\n', '\n')
    newtext = newtext.expandtabs(tabsize)
    if newtext == text:
        return
    f = open(filename, "w")
    f.write(newtext)
    f.close()
    print filename

# process()
    
def main():
    options, arguments = parse_invokation()
    top = os.path.abspath(arguments[0])
    extensions = arguments[1:]
    for root, directories, files in os.walk(top):
        for file in files:
            _, extension = os.path.splitext(file)
            if extension in extensions:
                process(os.path.join(root, file), options.tabs)

# main

if __name__ == "__main__":
    main()

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

from numpy import *

import sys
from PyQt4.Qwt3D._Qwt3D import *
from PyQt4.Qt import *

# enable all tracing options of the SIP generated bindings (requires -r option)
if False:
    import sip
    sip.settracemask(0x3f)


def matrix2d(nx, ny, minx, maxx, miny, maxy, function):
    """Return a data matrix to test the interface to the C++ member function 
    bool SurfacePlot::loadFromData(
        double **, unsigned int, unsigned int, double, double, double, double);
    """
    # columns
    xs = multiply.outer(
        minx + ((maxx-minx)/(nx-1))*arange(nx), ones(ny, float))
    # rows
    ys = multiply.outer(
        ones((nx,), float), miny+((maxy-miny)/(ny-1))*arange(ny))
    return function(xs, ys)

# matrix2d()


def matrix3d(nx, ny, minx, maxx, miny, maxy, function):
    """Return a data matrix to test the interface to the C++ member function
    bool SurfacePlot::loadFromData(
        Triple **, unsigned int, unsigned int, bool = false, bool = false);
    """
    xyzs = zeros((nx, ny, 3), float)
    # columns
    xyzs[:,:,0] = multiply.outer(
        minx + ((maxx-minx)/(nx-1))*arange(nx), ones(ny, float))
    # rows
    xyzs[:,:,1] = multiply.outer(
        ones((nx,), float), miny+((maxy-miny)/(ny-1))*arange(ny))
    # result
    xyzs[:,:,2] = function(xyzs[:,:,0], xyzs[:,:,1])
    return xyzs

# matrix3d()


def saddle(x, y):
    return x*y

# saddle()


class Plot(SurfacePlot):

    def __init__(self, *args):
        SurfacePlot.__init__(self, *args)
        # fonts
        family = QCoreApplication.instance().font().family()
        if 'Verdana' in QFontDatabase().families():
            family = 'Verdana'
        family = 'Courier'

        self.coordinates().setLabelFont(family, 14)
        self.coordinates().setNumberFont(family, 12)

        self.setBackgroundColor(RGBA(1.0, 1.0, 0.6))

        self.setRotation(30, 0, 15)
        self.setScale(1.0, 1.0, 1.0)

        nx, ny, minx, maxx, miny, maxy = 3, 5, -1.0, 1.0, -1.0, 1.0
        if True:
            zs = matrix2d(nx, ny, minx, maxx, miny, maxy, saddle)
            print(type(zs))
            print(zs)
            self.loadFromData(zs, minx, maxx, miny, maxy)
        else:
            xyzs = matrix3d(nx, ny, minx, maxx, miny, maxy, saddle)
            print(type(zs))
            print(xyzs)
            self.loadFromData(xyzs)

        axes = self.coordinates().axes # alias

        for axis, label in ((X1, "x"), (Y1, "y"), (Z1, "z")):
            axes[axis].setAutoScale(False)
            axes[axis].setMajors(5) # 6 major ticks
            axes[axis].setMinors(3) # 2 minor ticks
            axes[axis].setLabelString(label)

        self.setCoordinateStyle(BOX)
        self.coordinates().setGridLines(True, True)
        self.coordinates().setLineSmooth(True)

        self.updateData()
        self.updateGL()

    # __init__()

# class Plot


def make():
    demo = Plot()
    demo.show()
    # Matrox cards on Linux work better with a resize() after show()
    demo.resize(600, 400)
    return demo

# make()


def main(args):
    app = QApplication(args)
    demo = make()
    app.exec_()

# main()


# Admire
if __name__ == '__main__':
    main(sys.argv)


# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import time
from distutils.core import setup

name = 'PyQwt3D'
version = '0.1.7'
#version = '%04d%02d%02d' % (time.localtime()[:3])

long_description = """
PyQwt3D is a set of Python bindings for the QwtPlot3D C++ class library.
The QwtPlot3D library (http://qwtplot3d.sourceforge.net) extends the Qt
framework with widgets to visualize 3-dimensional data.
"""

setup(
    name             = name,
    version          = version,
    description      = "Python bindings for the QwtPlot3D library",
    url              = "http://pyqwt.sourceforge.net",
    author           = "Gerard Vermeulen",
    author_email     = "gerard.vermeulen@grenoble.cnrs.fr",
    license          = "GPL",
    long_description = long_description,
    platforms        = "Unix, Windows (MSVC), MacOS/X",
    )

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import inspect, sys
# iqt is part of PyQwt
from PyQt5.Qwt3D import Plot3D, save

get_input = input
# If this is Python 2, use raw_input()
if sys.version_info[:2] <= (2, 7):
    get_input = raw_input

def walk(p):
    if isinstance(p, Plot3D):
        yield p
    else:
        for c in p.children():
            for cc in walk(c):
               yield cc

# def walk()


def main():
    print(inspect.getsource(save))

    for demo in ['ParametricSurfaceDemo',
                 'SimplePlot',
                 'TestNumPy',
                 ]:
        result = __import__(demo).make()
        get_input("Is the demo looking HAPPY? ")

        for format in ('png', 'pdf', 'ps', 'eps', 'svg'):
            print('Saving %s.%s...' % (demo, format))
            if save(result, '%s.%s' % (demo, format), format):
                print('success')
            else:
                print('failure')

    for demo in ['AutoSwitch',
                 'EnrichmentDemo',
                 ]:
        result = __import__(demo).make()
        get_input("Is the demo looking HAPPY? ")

        for i, w in enumerate(walk(result)):
            for format in ('png', 'pdf', 'ps', 'eps', 'svg'):
                print('Saving %s%s.%s...' % (demo, i, format),)
                if save(w, '%s%s.%s' % (demo, i, format), format):
                    print('success')
                else:
                    print('failure')
            
# main()

if __name__ == '__main__':
    main()

# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import sys
from PyQt4.Qt import QApplication, QCoreApplication, QFont, QFontDatabase
from PyQt4.Qt import QSplitter, QTimer, SIGNAL, Qt
from PyQt4.Qwt3D._Qwt3D import *


class Saddle(Function):

    def __init__(self, *args):
        Function.__init__(self, *args)

        self.setDomain(-2.5, 2.5, -2.5, 2.5)
        self.setMaxZ(1.5)
        self.setMinZ(-1.5)
        self.setMesh(31,31)

    # __init__()
    
    def __call__(self, x, y):
        return x*y

    # __call__()

# class Saddle


class Hat(Function):

    def __init__(self, *args):
        Function.__init__(self, *args)

        self.setDomain(-1.5, 1.5, -1.5, 1.5)
        self.setMesh(41, 41)

    # __init__()
    
    def __call__(self, x, y):
        return 1.0/(x*x+y*y+0.3)

    # __call__()

# class Hat


class Plot(SurfacePlot):

    def __init__(self, parent, updateinterval):
        SurfacePlot.__init__(self, parent)
        # fonts
        family = QCoreApplication.instance().font().family()
        if 'Verdana' in QFontDatabase().families():
            family = 'Verdana'
        family = 'Courier'

        self.setTitleFont(family, 16, QFont.Bold)
            
        self.setRotation(30, 0, 15)
        self.setShift(0.1, 0, 0)
        self.setZoom(0.8)

        self.coordinates().setNumberFont(family, 8)
        
        axes = self.coordinates().axes # alias
        for axis in axes:
            axis.setMajors(7)
            axis.setMinors(4)

        axes[X1].setLabelString("x")
        axes[Y1].setLabelString("y")
        axes[Z1].setLabelString("z")
        axes[X2].setLabelString("x")
        axes[Y2].setLabelString("y")
        axes[Z2].setLabelString("z")
        axes[X3].setLabelString("x")
        axes[Y3].setLabelString("y")
        axes[Z3].setLabelString("z")
        axes[X4].setLabelString("x")
        axes[Y4].setLabelString("y")
        axes[Z4].setLabelString("z")

        timer = QTimer(self)
        self.connect(timer, SIGNAL('timeout()'), self.rotate)
        timer.start(updateinterval)
        
    # __init__()

    def rotate(self):
        self.setRotation(int(self.xRotation()+2) % 360,
                         int(self.yRotation()+2) % 360,
                         int(self.zRotation()+2) % 360)

    # rotate()

# class Plot()


def make():
    demo = QSplitter(Qt.Horizontal)

    plot1 = Plot(demo, 30)
    plot1.setFloorStyle(FLOORISO)
    plot1.setCoordinateStyle(BOX)
    saddle = Saddle(plot1)
    saddle.create()
    plot1.setTitle("Autoswitching axes")
    plot1.setBackgroundColor(RGBA(1.0, 1.0, 0.6))
    plot1.makeCurrent()
    plot1.updateData()
    plot1.updateGL()
    
    plot2 = Plot(demo, 80)
    plot2.setZoom(0.8)
    hat = Hat(plot2)
    hat.create()
    plot2.setPlotStyle(HIDDENLINE)
    plot2.setFloorStyle(FLOORDATA)
    plot2.setCoordinateStyle(FRAME)
    plot2.setBackgroundColor(RGBA(1.0, 1.0, 0.6))
    plot2.makeCurrent()
    plot2.updateData()
    plot2.updateGL()

    demo.resize(800, 400)
    demo.show()
                
    return demo

# make()


def main(args):
    app = QApplication(args)
    demo = make()
    app.exec_()

# main()


# Admire
if __name__ == '__main__':
    main(sys.argv)


# Local Variables: ***
# mode: python ***
# End: ***



#!/usr/bin/env python

import os
import re
import sys

def stamp(html):
    """Stamp a Python HTML documentation page with the SourceForge logo"""

    def replace(m):
        return ('<span class="release-info">%s '
                'Hosted on <a href="http://sourceforge.net">'
                '<img src="http://sourceforge.net/'
                'sflogo.php?group_id=82987&type=1" width="88" height="31"'
                'border="0" alt="SourceForge Logo"></a></span>' % m.group(1))

    mailRe = re.compile(r'<span class="release-info">(.*)</span>')

    return re.sub(mailRe, replace, html)

# stamp()

if __name__ == '__main__':
    for name in sys.argv[1:]:
        html = open(name, 'r').read()
        text = stamp(html)
        if text != html:
            os.remove(name)
            file = open(name, 'w')
            file.write(text)
            file.close()

# Local Variables: ***
# mode: python ***
# End: ***
